
#include <windows.h>  
#include "addpage.h"  
#include "commctrl.h"


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

HWND    hPropSheet  = NULL;
LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "PropSheet_AddPage"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      if ( IsWindow( hPropSheet ) && PropSheet_IsDialogMessage( hPropSheet, &msg ) )
         continue;

      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}



LRESULT CALLBACK PageProc( HWND hDlg, UINT message,        
                           WPARAM wParam, LPARAM lParam )
{
static NMHDR* nmhdr;

   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_NOTIFY :
               nmhdr = (NMHDR*)lParam;
               switch( nmhdr->code )
               {
                       // The user selected the Cancel button
                       // and the properties that has been applied
                       // needs to be restored.
                       //.........................................
                  case PSN_RESET :
                         // Reset data back to original state.
                         // ..................................
                         break;

                       // Cancel or Close Pressed.
                       //.........................
                  case PSN_QUERYCANCEL :
                         DestroyWindow( hPropSheet );
                         break;

                       // OK Button Pressed.
                       //...................
                  case PSN_APPLY :
                         DestroyWindow( hPropSheet );
                         break;

               }
               break;
   }

   return( FALSE );
}


VOID DisplayProperties( HWND hWnd )
{
   PROPSHEETPAGE   psp;
   PROPSHEETHEADER psh;
   HPROPSHEETPAGE  hpsp;

   // Create the property pages.
   //...........................
   psp.dwSize    = sizeof( PROPSHEETPAGE );
   psp.dwFlags   = PSP_DEFAULT;
   psp.hInstance = hInst;
   psp.pszTemplate = "InitialPage";
   psp.pfnDlgProc  = (DLGPROC)PageProc;
   hpsp = CreatePropertySheetPage( &psp );

   // Initialize the property sheet structure.
   //.........................................
   memset( &psh, 0, sizeof( PROPSHEETHEADER ) );
   psh.dwSize     = sizeof( PROPSHEETHEADER );
   psh.dwFlags    = PSH_USEICONID | PSH_MODELESS | PSH_NOAPPLYNOW;
   psh.hInstance  = hInst;
   psh.hwndParent = hWnd;
   psh.pszIcon    = "SMALL";
   psh.nPages     = 1;
   psh.phpage     = &hpsp;
   psh.pszCaption = "Properties";

   // Create a modeless property sheet.
   //..................................
   hPropSheet = (HWND)PropertySheet( &psh );
}


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HPROPSHEETPAGE hPage = NULL;

   switch( uMsg )
   {
      case WM_CREATE :
              InitCommonControls();
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST :
                        if ( !IsWindow( hPropSheet ) )
                           DisplayProperties( hWnd );
                        break;

                 case IDM_ADDPAGE :
                        if ( IsWindow( hPropSheet ) && !hPage )
                        {
                           PROPSHEETPAGE psp;

                           // Create a new page.
                           //...................
                           psp.dwSize    = sizeof( PROPSHEETPAGE );
                           psp.dwFlags   = PSP_DEFAULT;
                           psp.hInstance = hInst;
                           psp.pszTemplate = "AddedPage";
                           psp.pfnDlgProc  = (DLGPROC)PageProc;
                           hPage = CreatePropertySheetPage( &psp );

                           // Add the page to the property sheet.
                           //....................................
                           PropSheet_AddPage( hPropSheet, hPage );
                        }
                        break; 

                 case IDM_REMPAGE :
                        if ( IsWindow( hPropSheet ) && hPage )
                        {
                           // Remove the page to the property sheet.
                           //.......................................
                           PropSheet_RemovePage( hPropSheet, 1, hPage );
                           hPage = NULL;
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);

              if ( hPage )
                 DestroyPropertySheetPage( hPage );

              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
